"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Field = void 0;
const scalar_1 = require("../constants/scalar");
class Field {
    constructor(name, type, isEnum, noNullArrayValues, noNull, isArray, directives, args, isDeprecated) {
        this.name = name;
        this.type = type;
        this.isEnum = isEnum;
        this.noNullArrayValues = noNullArrayValues;
        this.noNull = noNull;
        this.isArray = isArray;
        this.directives = directives;
        this.arguments = args;
        this.isDeprecated = isDeprecated;
    }
    setNoNull() {
        this.noNull = true;
    }
    /**
     * Set up types fields to handle tracking of foreign key that might have been added by other types
     * Init Object type parameters . obj = {key1 : value1, key2 : value2 ....}
     * @returns nothing
     */
    initObjectParameters() {
        this.foreign_key = null;
        // if the field is a relation
        this.relation = false;
        // if the field is added or is adding to another field
        this.delegated_field = {
            "state": false,
            "side": null,
            "associatedWith": {
                "type": null,
                "fieldName": null
            }
        };
        // if the field will appear in final model (tables) ex for oneToMany relation the field may dissapear
        this.in_model = true;
        // contains info if the field will be in a joinTable in final model, the name of the table
        // the name of the fields associated in the table
        this.joinTable = {
            "state": false,
            "name": null,
            "contains": []
        };
        //Contains info about OneToOne relations
        this.oneToOneInfo = null;
        // adds info about the field sqlType
        this.sqlType = getSQLType(this.type);
    }
}
exports.Field = Field;
const getSQLType = (fieldType) => {
    switch (fieldType) {
        case "ID":
        case "Int":
        case scalar_1.Scalars.UtcOffset:
        case scalar_1.Scalars.EmailAddress:
        case scalar_1.Scalars.URL:
        case scalar_1.Scalars.PhoneNumber:
        case scalar_1.Scalars.PostalCode:
        case scalar_1.Scalars.HexColorCode:
        case scalar_1.Scalars.HSL:
        case scalar_1.Scalars.HSLA:
        case scalar_1.Scalars.NonPositiveInt:
        case scalar_1.Scalars.PositiveInt:
        case scalar_1.Scalars.NonNegativeInt:
        case scalar_1.Scalars.NegativeInt:
        case scalar_1.Scalars.UnsignedInt:
        case scalar_1.Scalars.RGB:
            return "int";
        case "String":
        case scalar_1.Scalars.ISBN:
        case scalar_1.Scalars.RGBA:
            return "text";
        case "Boolean":
            return "boolean";
        case "Float":
        case scalar_1.Scalars.NonPositiveFloat:
        case scalar_1.Scalars.PositiveFloat:
        case scalar_1.Scalars.NonNegativeFloat:
        case scalar_1.Scalars.NegativeFloat:
        case scalar_1.Scalars.UnsignedFloat:
            return "float8";
        case scalar_1.Scalars.Date:
            return "date";
        case scalar_1.Scalars.Time:
            return "time";
        case scalar_1.Scalars.DateTime:
            return "timestamp";
        case scalar_1.Scalars.BigInt:
        case scalar_1.Scalars.Long:
        case scalar_1.Scalars.Port:
            return "int8";
        case scalar_1.Scalars.GUID:
            return "uuid";
        case scalar_1.Scalars.IPv4:
        case scalar_1.Scalars.IPv6:
            return "inet";
        case scalar_1.Scalars.MAC:
            return "macaddr";
        case scalar_1.Scalars.USCurrency:
        case scalar_1.Scalars.Currency:
            return "money";
        case scalar_1.Scalars.JSON:
        case scalar_1.Scalars.JSONObject:
            return "json";
        case scalar_1.Scalars.Byte:
            return "bytea";
        case scalar_1.Scalars.LineString:
        case scalar_1.Scalars.Point:
        case scalar_1.Scalars.Polygon:
            return "geometry";
        default:
            return "int";
    }
};
