"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Type = void 0;
const field_1 = require("./field");
const get_sql_table_name_1 = require("../utils/get-sql-table-name");
class Type {
    constructor(type, typeName, sqlTypeName, description, values, directives, implementedTypes) {
        this.type = type;
        this.typeName = typeName;
        this.sqlTypeName = sqlTypeName;
        this.description = description;
        this.directives = directives;
        this.values = values;
        this.fields = [];
        this.implementedTypes = implementedTypes;
        // stores all the relations which were found for a given type
        this.relationList = [];
    }
    static initTypes(schemaJSON) {
        let types = [];
        for (const typeName in schemaJSON) {
            let typeToAdd = new Type(schemaJSON[typeName].type, typeName, (0, get_sql_table_name_1.getSQLTableName)(typeName), schemaJSON[typeName].description ? schemaJSON[typeName].description : "", schemaJSON[typeName].values ? schemaJSON[typeName].values : [], schemaJSON[typeName].directives ? schemaJSON[typeName].directives : [], schemaJSON[typeName].implementedTypes ? schemaJSON[typeName].implementedTypes : []);
            for (const field of schemaJSON[typeName].fields) {
                let fieldToAdd = new field_1.Field(field.name, field.type, field.isEnum, field.noNullArrayValues, field.noNull, field.isArray, field.directives, field.arguments, field.isDeprecated);
                if (field["name"] === 'id') {
                    fieldToAdd.setNoNull();
                }
                //Init missingInfos on the field
                fieldToAdd.initObjectParameters();
                //adds the field to the type
                typeToAdd.addField(fieldToAdd);
            }
            types.push(typeToAdd);
        }
        // change isEnum attribute of field if it refers to an Enumeation Type
        types.filter(type => type.type === "ObjectTypeDefinition")
            .forEach(currentType => {
            currentType.fields.forEach(field => {
                if (types.find(typeE => field.type === typeE.typeName && typeE.type === "EnumTypeDefinition"))
                    field.isEnum = true;
                else
                    field.isEnum = false;
            });
        });
        return types;
    }
    addField(field) {
        this.fields.push(field);
    }
    isNotOperation() {
        return this.typeName !== "Query" && this.typeName !== "Mutation" && this.typeName !== "Subscription";
    }
}
exports.Type = Type;
