"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInitEachFieldsModelsJS = exports.getInitEachModelsJS = exports.getAllTables = void 0;
const scalar_1 = require("../constants/scalar");
const manage_scalars_1 = require("../scalar-managment/manage-scalars");
/** Fonctions principales */
/** DATABASE (tables, init, fill, drop) */
// Tables
// Get all the tables, with columns, based on the types we have
const getAllTables = (types) => {
    let allTables = [];
    let typesNameArray = types.map(type => type.typeName);
    types.forEach(currentType => {
        let tableTemp = [];
        // Fill up the infos on scalar field (int, string, etc.)
        if (currentType.isNotOperation() && currentType.type === "ObjectTypeDefinition") {
            //get scalar field infos
            tableTemp.push(...getScalarFieldInfo(currentType, typesNameArray));
            allTables.push({
                name: currentType.typeName,
                sqlName: currentType.sqlTypeName,
                columns: tableTemp,
                isJoinTable: false
            });
        }
    });
    return allTables;
};
exports.getAllTables = getAllTables;
const getInitEachModelsJS = (tables) => {
    let s = '';
    tables.forEach(table => {
        s += 'init' + table.name + '()\n';
    });
    return s;
};
exports.getInitEachModelsJS = getInitEachModelsJS;
const getInitEachFieldsModelsJS = (types) => {
    let s = '';
    s += getInitEachModelsFields(types);
    s += '\n\n';
    return s;
};
exports.getInitEachFieldsModelsJS = getInitEachFieldsModelsJS;
/** Fonctions utilitaires */
const getInitEachModelsFields = (types) => {
    let s = '';
    types.forEach(type => {
        if (type.isNotOperation()) {
            let nameList = type.typeName.toLowerCase();
            s += 'for(let i = 0; i < ' + nameList + 'Tab.length; i++){\n\t' + nameList + 'Tab[i] = update' + type.typeName + '(' + nameList + 'Tab[i], i);\n}';
        }
    });
    return s;
};
const pushBasicScalarFieldInfo = (currentType, field, tableTemp) => {
    switch (field.type) {
        case "ID":
            tableTemp.push({
                field: "Pk_" + currentType.sqlTypeName + "_id",
                fieldType: "int",
                unique: false,
                constraint: "SERIAL PRIMARY KEY NOT NULL",
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "int";
            break;
        case "String":
            tableTemp.push({
                field: field.name,
                fieldType: "text",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "text";
            break;
        case "Int":
            tableTemp.push({
                field: field.name,
                fieldType: "int",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "int";
            break;
        case "Boolean":
            tableTemp.push({
                field: field.name,
                fieldType: "boolean",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "boolean";
            break;
        case "Float":
            tableTemp.push({
                field: field.name,
                fieldType: "float8",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "float8";
            break;
        default:
            let fkInfo = field.foreign_key;
            tableTemp.push({
                field: fkInfo.name,
                fieldType: fkInfo.type,
                noNull: fkInfo.noNull,
                unique: false,
                constraint: fkInfo.constraint,
                isArray: fkInfo.isArray,
                gqlType: fkInfo.type,
                noNullArrayValues: field.noNullArrayValues
            });
            break;
    }
};
function pushCustomScalarFieldInfo(currentType, field, tableTemp) {
    switch (field.type) {
        case scalar_1.Scalars.Date:
            tableTemp.push({
                field: field.name,
                fieldType: "date",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "date";
            break;
        case scalar_1.Scalars.Time:
            tableTemp.push({
                field: field.name,
                fieldType: "time",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "time";
            break;
        case scalar_1.Scalars.DateTime:
            tableTemp.push({
                field: field.name,
                fieldType: "timestamp",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "timestamp";
            break;
        case scalar_1.Scalars.NonPositiveFloat:
        case scalar_1.Scalars.PositiveFloat:
        case scalar_1.Scalars.NonNegativeFloat:
        case scalar_1.Scalars.NegativeFloat:
        case scalar_1.Scalars.UnsignedFloat:
            tableTemp.push({
                field: field.name,
                fieldType: "float8",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "float8";
            break;
        case scalar_1.Scalars.BigInt:
        case scalar_1.Scalars.Long:
        case scalar_1.Scalars.Port:
            tableTemp.push({
                field: field.name,
                fieldType: "int8",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "int8";
            break;
        case scalar_1.Scalars.GUID:
            tableTemp.push({
                field: field.name,
                fieldType: "uuid",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "uuid";
            break;
        case scalar_1.Scalars.IPv4:
        case scalar_1.Scalars.IPv6:
            tableTemp.push({
                field: field.name,
                fieldType: "inet",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "inet";
            break;
        case scalar_1.Scalars.MAC:
            tableTemp.push({
                field: field.name,
                fieldType: "macaddr",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "macaddr";
            break;
        case scalar_1.Scalars.USCurrency:
        case scalar_1.Scalars.Currency:
            tableTemp.push({
                field: field.name,
                fieldType: "money",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "money";
            break;
        case scalar_1.Scalars.JSON:
        case scalar_1.Scalars.JSONObject:
            tableTemp.push({
                field: field.name,
                fieldType: "json",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "json";
            break;
        case scalar_1.Scalars.Byte:
            tableTemp.push({
                field: field.name,
                fieldType: "bytea",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "bytea";
            break;
        case scalar_1.Scalars.LineString:
        case scalar_1.Scalars.Point:
        case scalar_1.Scalars.Polygon:
            tableTemp.push({
                field: field.name,
                fieldType: "geometry",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "geometry";
            break;
        case scalar_1.Scalars.UtcOffset:
        case scalar_1.Scalars.EmailAddress:
        case scalar_1.Scalars.URL:
        case scalar_1.Scalars.PhoneNumber:
        case scalar_1.Scalars.PostalCode:
        case scalar_1.Scalars.HexColorCode:
        case scalar_1.Scalars.HSL:
        case scalar_1.Scalars.HSLA:
        case scalar_1.Scalars.NonPositiveInt:
        case scalar_1.Scalars.PositiveInt:
        case scalar_1.Scalars.NonNegativeInt:
        case scalar_1.Scalars.NegativeInt:
        case scalar_1.Scalars.UnsignedInt:
        case scalar_1.Scalars.RGB:
            tableTemp.push({
                field: field.name,
                fieldType: "int",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "int";
            break;
        case scalar_1.Scalars.ISBN:
        case scalar_1.Scalars.RGBA:
        default:
            // By default, scalar type other than the ones that have specific column type in postgres, it's up to the final user to modify the final field type in the table
            tableTemp.push({
                field: field.name,
                fieldType: "text",
                unique: false,
                constraint: null,
                isArray: field.isArray,
                gqlType: field.type,
                noNull: field.noNull,
                noNullArrayValues: field.noNullArrayValues
            });
            field.sqlType = "text";
            break;
    }
}
const getScalarFieldInfo = (currentType, typesNameArray) => {
    let tableTemp = [];
    currentType.fields.forEach(currentField => {
        if (!typesNameArray.includes(currentField.type) && currentField["in_model"] && (0, manage_scalars_1.isBasicScalar)(currentField.type)) {
            pushBasicScalarFieldInfo(currentType, currentField, tableTemp);
        }
        else if (currentField.type in scalar_1.Scalars) {
            pushCustomScalarFieldInfo(currentType, currentField, tableTemp);
        }
        else { // handle types who are traduced into foreignKeys
            if (currentField.foreign_key !== null && currentField.in_model) {
                let fkInfo = currentField.foreign_key;
                tableTemp.push({
                    field: fkInfo.name,
                    fieldType: fkInfo.type,
                    unique: false,
                    constraint: currentField.type === "ID" ? "SERIAL PRIMARY KEY" : null,
                    isArray: currentField.isArray,
                    gqlType: currentField.type,
                    noNull: currentField.noNull,
                    noNullArrayValues: currentField.noNullArrayValues
                });
            }
        }
    });
    return tableTemp;
};
