"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getQuerySelfJoinMany = exports.getQuerySelfJoinOne = exports.getJoinTables = exports.getRelations = void 0;
/** Fonctions principales */
const relationships_1 = require("../constants/relationships");
const get_sql_table_name_1 = require("../utils/get-sql-table-name");
const manage_scalars_1 = require("../scalar-managment/manage-scalars");
const one_to_one_not_allowed_1 = require("./error/one-to-one-not-allowed");
/**
 *  Compute relationships oneToMany, manyToMany, etc..
 * @param {*} types contains all types with associated attributes read from easygraphql-parser
 * @returns
 */
const getRelations = (types) => {
    let manyToMany = [];
    let targetSQLTypeName;
    let currentSQLTypeName;
    let relatedFieldName;
    let targetType;
    types.filter(type => type.type === "ObjectTypeDefinition" && type.isNotOperation())
        .forEach(currentType => {
        let relationalFields = currentType.fields.filter(field => !(0, manage_scalars_1.isScalar)(field.type));
        relationalFields.forEach(relationalField => {
            let inn = relationalField.isArray ? 2 : 1;
            let out = getRelationOf(relationalField.type, types, currentType.typeName);
            switch (true) { //Switch true to check multiple arguments
                /** OneOnly relationships **/
                case inn === 1 && out === 0:
                    currentType.relationList.push({
                        type: relationalField.type,
                        relation: relationships_1.Relationships.oneOnly
                    });
                    targetSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(relationalField.type);
                    relationalField.relation = true;
                    relationalField.relationType = relationships_1.Relationships.oneOnly;
                    relationalField.foreign_key = {
                        name: `Fk_${relationalField.name}_${targetSQLTypeName}_id`,
                        type: "int",
                        noNull: relationalField.noNull,
                        isForeignKey: true,
                        constraint: `FOREIGN KEY ("Fk_${relationalField.name}_${targetSQLTypeName}_id") REFERENCES "${targetSQLTypeName}" ("Pk_${targetSQLTypeName}_id")`
                    };
                    break;
                /** OneToOne || SelfJoinOne relationships **/
                case inn === 1 && out === 1:
                    if (relationalField.type === currentType.typeName) {
                        currentType.relationList.push({
                            type: relationalField.type,
                            relation: relationships_1.Relationships.selfJoinOne
                        });
                        targetSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(relationalField.type);
                        relationalField.relation = true;
                        relationalField.relationType = relationships_1.Relationships.selfJoinOne;
                        relationalField.foreign_key = {
                            name: `Fk_${relationalField.name}_${targetSQLTypeName}_id`,
                            type: "int",
                            noNull: relationalField.noNull,
                            isForeignKey: true,
                            constraint: `FOREIGN KEY ("Fk_${relationalField.name}_${targetSQLTypeName}_id") REFERENCES "${targetSQLTypeName}" ("Pk_${targetSQLTypeName}_id")`
                        };
                    }
                    else {
                        let targetedType = types.find(type => type.typeName === relationalField.type);
                        let targetField = targetedType === null || targetedType === void 0 ? void 0 : targetedType.fields.find(field => field.type === currentType.typeName);
                        if ((targetField === null || targetField === void 0 ? void 0 : targetField.noNull) && relationalField.noNull) {
                            throw new one_to_one_not_allowed_1.OneToOneRelationNotAllowedError();
                        }
                        currentType.relationList.push({
                            type: relationalField.type,
                            relation: relationships_1.Relationships.oneToOne
                        });
                        if (!(targetField === null || targetField === void 0 ? void 0 : targetField.oneToOneInfo)) {
                            targetSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(relationalField.type);
                            relationalField.relation = true;
                            relationalField.relationType = relationships_1.Relationships.oneToOne;
                            relationalField.foreign_key = {
                                name: `Fk_${relationalField.name}_${targetSQLTypeName}_id`,
                                type: "int",
                                noNull: relationalField.noNull,
                                isForeignKey: true,
                                constraint: `FOREIGN KEY ("Fk_${relationalField.name}_id") REFERENCES "${targetSQLTypeName}"`
                            };
                            relationalField.oneToOneInfo = `Fk_${relationalField.name}_${targetSQLTypeName}_id`;
                        }
                    }
                    break;
                /** OneToMany relationships **/
                case inn === 1 && out === 2:
                    relatedFieldName = getRelatedFieldName(currentType.typeName, types, relationalField.type);
                    currentType.relationList.push({
                        type: relationalField.type,
                        relation: relationships_1.Relationships.oneToMany,
                        relatedFieldName,
                    });
                    targetSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(relationalField.type);
                    relationalField.relation = true;
                    relationalField.relationType = relationships_1.Relationships.oneToMany;
                    relationalField.foreign_key = {
                        name: `Fk_${relationalField.name}_${targetSQLTypeName}_id`,
                        type: "int",
                        noNull: relationalField.noNull,
                        isForeignKey: true,
                        constraint: `FOREIGN KEY ("Fk_${relationalField.name}_${targetSQLTypeName}_id") REFERENCES "${targetSQLTypeName}" ("Pk_${targetSQLTypeName}_id")`
                    };
                    break;
                /** ManyOnly relationships **/
                case inn === 2 && out === 0:
                    currentType.relationList.push({
                        type: relationalField.type,
                        relation: relationships_1.Relationships.manyOnly
                    });
                    targetSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(relationalField.type);
                    currentSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(currentType.typeName);
                    relationalField.relation = true;
                    relationalField.relationType = relationships_1.Relationships.manyOnly;
                    // Foreign key field that will be added to the target type
                    let addedForeignKeyField = JSON.parse(JSON.stringify(relationalField));
                    // delegate object completion
                    relationalField.delegated_field.side = "origin";
                    relationalField.delegated_field.associatedWith.type = relationalField.type;
                    relationalField.delegated_field.associatedWith.fieldName = `Fk_${relationalField.name}_${currentSQLTypeName}_id`;
                    // Field does not appear in model
                    relationalField.in_model = false;
                    addedForeignKeyField.name = `Fk_${relationalField.name}_${currentSQLTypeName}_id`;
                    addedForeignKeyField.type = "Int";
                    addedForeignKeyField.isArray = false;
                    addedForeignKeyField.noNull = relationalField.noNullArrayValues;
                    addedForeignKeyField.foreign_key = {
                        name: `Fk_${relationalField.name}_${currentSQLTypeName}_id`,
                        type: "int",
                        noNull: relationalField.noNullArrayValues,
                        isForeignKey: true,
                        constraint: `FOREIGN KEY ("Fk_${relationalField.name}_${currentSQLTypeName}_id") REFERENCES "${currentSQLTypeName}" ("Pk_${currentSQLTypeName}_id")`
                    };
                    addedForeignKeyField.delegated_field.state = true;
                    addedForeignKeyField.delegated_field.side = "target";
                    addedForeignKeyField.delegated_field.associatedWith.type = currentType.typeName;
                    addedForeignKeyField.delegated_field.associatedWith.fieldName = relationalField.name;
                    //targetType = types.find(type => type.typeName === relationalField.type)
                    //targetType?.fields.push(addedForeignKeyField);
                    break;
                /** ManyToOne **/
                case inn === 2 && out === 1:
                    relatedFieldName = getRelatedFieldName(currentType.typeName, types, relationalField.type);
                    currentType.relationList.push({
                        type: relationalField.type,
                        relation: relationships_1.Relationships.manyToOne,
                        relatedFieldName,
                    });
                    relationalField.relation = true;
                    relationalField.relationType = relationships_1.Relationships.manyToOne;
                    relationalField.in_model = false;
                    break;
                /** ManyToMany | SelfJoinMany **/
                case inn === 2 && out === 2:
                    targetSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(relationalField.type);
                    currentSQLTypeName = (0, get_sql_table_name_1.getSQLTableName)(currentType.typeName);
                    if (relationalField.type === currentType.typeName) {
                        currentType.relationList.push({
                            type: relationalField.type,
                            relation: relationships_1.Relationships.selfJoinMany
                        });
                        relationalField.relationType = relationships_1.Relationships.selfJoinMany;
                        relationalField.joinTable.state = true;
                        relationalField.joinTable.name = `${currentType.typeName}_${currentType.typeName}_${relationalField.name}`;
                        relationalField.joinTable.contains.push({
                            fieldName: `${currentSQLTypeName}1_id`,
                            type: relationalField.type,
                            constraint: `FOREIGN KEY ("${currentSQLTypeName}1_id") REFERENCES "${targetSQLTypeName}" ("Pk_${targetSQLTypeName}_id")`
                        });
                        relationalField.joinTable.contains.push({
                            fieldName: `${currentSQLTypeName}2_id`,
                            type: currentType.typeName,
                            constraint: `FOREIGN KEY ("${currentSQLTypeName}2_id") REFERENCES "${currentSQLTypeName}" ("Pk_${currentSQLTypeName}_id")`
                        });
                    }
                    else {
                        currentType.relationList.push({
                            type: relationalField.type,
                            relation: relationships_1.Relationships.manyToMany
                        });
                        relationalField.relationType = relationships_1.Relationships.manyToMany;
                        targetType = types.find(type => type.typeName === relationalField.type);
                        let targetField = targetType === null || targetType === void 0 ? void 0 : targetType.fields.find(field => field.type === currentType.typeName);
                        // If the joinTable exists from the other entity we don't need to create a new one
                        let alreadyDefinedRelation = manyToMany.filter(relation => relation.field === (targetField === null || targetField === void 0 ? void 0 : targetField.name) && relationalField.name === relation.target);
                        if (alreadyDefinedRelation[0]) {
                            relationalField.joinTable = targetField === null || targetField === void 0 ? void 0 : targetField.joinTable;
                        }
                        else {
                            let jointTableName = `${currentType.typeName}_${relationalField.type}_${relationalField.name}`;
                            manyToMany.push({
                                name: jointTableName,
                                field: relationalField.name,
                                target: targetField === null || targetField === void 0 ? void 0 : targetField.name
                            });
                            relationalField.joinTable.state = true;
                            relationalField.joinTable.name = jointTableName;
                            relationalField.joinTable.contains.push({
                                fieldName: `${targetSQLTypeName}_id`,
                                type: relationalField.type,
                                constraint: `FOREIGN KEY ("${targetSQLTypeName}_id") REFERENCES "${targetSQLTypeName}" ("Pk_${targetSQLTypeName}_id")`
                            });
                            relationalField.joinTable.contains.push({
                                fieldName: `${currentSQLTypeName}_id`,
                                type: currentType.typeName,
                                constraint: `FOREIGN KEY ("${currentSQLTypeName}_id") REFERENCES "${currentSQLTypeName}" ("Pk_${currentSQLTypeName}_id")`
                            });
                        }
                    }
                    relationalField.relation = true;
                    relationalField.in_model = false;
                    break;
            }
        });
    });
    return types;
};
exports.getRelations = getRelations;
/**
 * Build the list of join tables to add to the schema on top of standard object
 *
 * @returns Tables description
 * @param types
 */
const getJoinTables = (types) => {
    let results = [];
    types.forEach(currentType => {
        if (currentType.isNotOperation()) {
            currentType.fields.filter(field => field.relationType == relationships_1.Relationships.selfJoinMany || field.relationType == relationships_1.Relationships.manyToMany && field.joinTable.state)
                .forEach(relationalField => {
                //Check if jointable already existing
                let alreadyExisting = results.filter(result => result.name === relationalField.joinTable.name);
                if (alreadyExisting[0]) {
                    return;
                }
                let joinTableColumns = relationalField.joinTable.contains.map(column => {
                    return {
                        field: column.fieldName,
                        fieldType: 'int',
                        constraint: column.constraint
                    };
                });
                results.push({
                    name: relationalField.joinTable.name,
                    sqlName: relationalField.joinTable.name.toLowerCase(),
                    columns: joinTableColumns,
                    isJoinTable: true
                });
            });
        }
    });
    return results;
};
exports.getJoinTables = getJoinTables;
const getQuerySelfJoinOne = (currentTypeName, fields) => {
    let result = "";
    fields.forEach(field => {
        if (field.type === currentTypeName) {
            result += "SELECT * FROM \"" + currentTypeName + "\" as t1 WHERE t1.\"Pk_Employe_id\" = (SELECT \"" + field.name.toLowerCase() + "_id\" FROM  \"" + currentTypeName + "\" WHERE \"" + currentTypeName + "\".\"Pk_" + currentTypeName + "_id\" = :value)";
        }
    });
    return result;
};
exports.getQuerySelfJoinOne = getQuerySelfJoinOne;
const getQuerySelfJoinMany = (currentTypeName, fields) => {
    let result = "";
    fields.forEach(field => {
        if (field.type === currentTypeName) {
            result += "SELECT t2.* FROM \"" + currentTypeName + "\" as t1 LEFT OUTER JOIN \"" + currentTypeName + "_" + field.name.toLowerCase() + "\" as joint ON t1.\"Pk_" + currentTypeName + "_id\" = joint.\"" + currentTypeName.toLowerCase() + "_id\" LEFT OUTER JOIN \"" + currentTypeName + "\" as t2 ON joint." + field.name.toLowerCase() + "_id = t2.\"Pk_" + currentTypeName + "_id\" WHERE t1.\"Pk_" + currentTypeName + "_id\" = :value '+sorting+' '+limit+' '+offset";
        }
    });
    return result;
};
exports.getQuerySelfJoinMany = getQuerySelfJoinMany;
/** Fonctions utilitaires */
/**
 *
 * @param {*} fields Fields of the targeted object
 * @param {*} currentTypeName TypeName being inspected
 * @returns 2 if relationship is [Type], 1 if relationship is Type, 0 if no relationship
 * @description Doesn't work if there is several time the same type referenced in the fileds !
 */
const getManyOrOne = (fields, currentTypeName) => {
    for (let field of fields) {
        if (field.type === currentTypeName) {
            return field.isArray ? 2 : 1;
        }
    }
    return 0;
};
/**
 *
 * @param {*} targetTypeName : Target type for the relation
 * @param {*} types : Types defined in the schema
 * @param {*} currentTypeName : Current Type being processed
 * @returns : 2 if relationship is [Type], 1 if relationship is Type, 0 if no relationship
 */
const getRelationOf = (targetTypeName, types, currentTypeName) => {
    for (let type of types) {
        if (type.typeName === targetTypeName) {
            return getManyOrOne(type.fields, currentTypeName);
        }
    }
    return 0;
};
/**
 *
 * @param {*} targetTypeName : Target type for the relation
 * @param {*} types : Types defined in the schema
 * @param {*} currentTypeName : Current Type being processed
 * @returns : 2 if relationship is [Type], 1 if relationship is Type, 0 if no relationship
 */
const getRelatedFieldName = (currentTypeName, types, relatedFieldType) => {
    const relatedType = types.find(type => type.typeName === relatedFieldType);
    const relatedField = relatedType === null || relatedType === void 0 ? void 0 : relatedType.fields.find(field => field.type.toLowerCase().includes(currentTypeName.toLowerCase()));
    return (relatedField === null || relatedField === void 0 ? void 0 : relatedField.name);
};
