"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typesGenerator = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const index_1 = require("../index");
const typesGenerator = (schemaJSON) => {
    let types = index_1.Type.initTypes(schemaJSON);
    let isValidSchema = (0, index_1.isSchemaValid)(types);
    if (!isValidSchema.response) {
        if (isValidSchema.reason === "Missing required id field of type ID in one or multiple Entity") {
            let idField = JSON.parse(fs_1.default.readFileSync(path_1.default.join(__dirname, 'id-field.json'), 'utf8'));
            types
                .filter(type => type.type === "ObjectTypeDefinition" && type.isNotOperation())
                .forEach(type => {
                console.log(`type ${type.typeName} is missing ID field, one was created automatically`);
                if (!(0, index_1.hasFieldType)(type, 'ID').answers) {
                    type.fields.unshift(idField);
                }
            });
        }
        else {
            throw new Error('Incorrect schema, please write a valid graphql schema based on the supported guidelines.\nReason: ' +
                isValidSchema.reason);
        }
    }
    console.log('Your schema is valid and is containing the following types: ');
    types.forEach(type => console.log(`name : ${type.typeName}, `
        + `fields : ${type.fields.map(field => field.name)}\n`));
    return types;
};
exports.typesGenerator = typesGenerator;
